<?php

namespace Drupal\static_node\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Pager\PagerManagerInterface;

/**
 * Controller for the static files management page.
 */
class StaticFileManagementController extends ControllerBase {

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $pathAliasManager;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;
  
  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;
  
  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;
  
  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * Constructs a StaticFileManagementController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\path_alias\AliasManagerInterface $path_alias_manager
   *   The path alias manager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pager_manager
   *   The pager manager.
   */
  public function __construct(
    Connection $database,
    EntityTypeManagerInterface $entity_type_manager,
    AliasManagerInterface $path_alias_manager,
    FileUrlGeneratorInterface $file_url_generator,
    DateFormatterInterface $date_formatter,
    RequestStack $request_stack,
    PagerManagerInterface $pager_manager
  ) {
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
    $this->pathAliasManager = $path_alias_manager;
    $this->fileUrlGenerator = $file_url_generator;
    $this->dateFormatter = $date_formatter;
    $this->requestStack = $request_stack;
    $this->pagerManager = $pager_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('entity_type.manager'),
      $container->get('path_alias.manager'),
      $container->get('file_url_generator'),
      $container->get('date.formatter'),
      $container->get('request_stack'),
      $container->get('pager.manager')
    );
  }

  /**
   * Displays static files management page.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return array
   *   A render array for the management page.
   */
  public function overview(Request $request) {
    // Build the filter form
    $filters = $this->buildFiltersForm($request);
    
    $header = [
      'file' => [
        'data' => $this->t('File'),
        'field' => 'filename',
        'sort' => 'asc',
      ],
      'node' => $this->t('Content'),
      'type' => [
        'data' => $this->t('Type'),
        'field' => 'n.type',
      ],
      'changed' => [
        'data' => $this->t('Last Updated'),
        'field' => 'f.changed',
        'sort' => 'desc',
      ],
      'size' => [
        'data' => $this->t('Size'),
        'field' => 'f.filesize',
      ],
      'operations' => $this->t('Operations'),
    ];

    // Get static files with applied filters
    $items = $this->getStaticFiles($header);
    
    $build['filters'] = $filters;
    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => [],
      '#empty' => $this->t('No static files found.'),
      '#attributes' => [
        'class' => ['static-files-listing'],
      ],
    ];

    foreach ($items as $item) {
      $file_url = $this->fileUrlGenerator->generateAbsoluteString($item->uri);
      
      // Make sure we have a valid node ID before trying to generate a URL
      $node_link = NULL;
      if (!empty($item->nid)) {
        $node_url = Url::fromRoute('entity.node.canonical', ['node' => $item->nid])->toString();
        $node_link = [
          'data' => [
            '#type' => 'link',
            '#title' => $item->title ?? $this->t('Unknown'),
            '#url' => Url::fromUri('internal:' . $node_url),
          ],
        ];
      } else {
        $node_link = [
          'data' => $item->title ?? $this->t('N/A'),
        ];
      }
      
      $operations = [
        'data' => [
          '#type' => 'operations',
          '#links' => [
            'view' => [
              'title' => $this->t('View'),
              'url' => Url::fromUri($file_url, ['attributes' => ['target' => '_blank']]),
            ],
            'delete' => [
              'title' => $this->t('Delete'),
              'url' => Url::fromRoute('static_node.delete_static_file', ['file' => $item->fid]),
            ],
          ],
        ],
      ];
      
      // Only add the regenerate option if we have a valid node ID
      if (!empty($item->nid)) {
        $operations['data']['#links']['regenerate'] = [
          'title' => $this->t('Regenerate'),
          'url' => Url::fromRoute('static_node.regenerate_static_file', ['node' => $item->nid]),
        ];
      }
      
      $build['table']['#rows'][] = [
        'file' => [
          'data' => [
            '#type' => 'link',
            '#title' => $item->filename,
            '#url' => Url::fromUri($file_url, ['attributes' => ['target' => '_blank']]),
          ],
        ],
        'node' => $node_link,
        'type' => $item->type_label ?? $this->t('Unknown'),
        'changed' => $this->dateFormatter->format($item->changed, 'short'),
        'size' => $this->formatBytes($item->filesize),
        'operations' => $operations,
      ];
    }
    
    // Add pager
    $build['pager'] = [
      '#type' => 'pager',
    ];

    return $build;
  }

  /**
   * Builds the filters form.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return array
   *   A renderable form array.
   */
  protected function buildFiltersForm(Request $request) {
    // Get node types
    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $type_options = ['' => $this->t('- Any -')];
    foreach ($types as $type) {
      $type_options[$type->id()] = $type->label();
    }

    $filters = [];
    
    $filters['filters'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['form--inline', 'clearfix']],
    ];
    
    $filters['filters']['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#default_value' => $request->query->get('title'),
      '#size' => 25,
    ];
    
    $filters['filters']['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Type'),
      '#options' => $type_options,
      '#default_value' => $request->query->get('type'),
    ];
    
    $filters['filters']['actions'] = [
      '#type' => 'actions',
    ];
    
    $filters['filters']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];
    
    $filters['filters']['actions']['reset'] = [
      '#type' => 'link',
      '#title' => $this->t('Reset'),
      '#url' => Url::fromRoute('static_node.files_management'),
      '#attributes' => ['class' => ['button']],
    ];
    
    $filters['#method'] = 'get';
    
    return $filters;
  }

  /**
   * Retrieves static files from the database based on filters.
   *
   * @param array $header
   *   The table header for sorting.
   *
   * @return \stdClass[]
   *   An array of static file data objects.
   */
  protected function getStaticFiles(array $header) {
    // Start with a simple query for files matching our pattern
    $query = $this->database->select('file_managed', 'f')
      ->fields('f', ['fid', 'filename', 'uri', 'filesize', 'changed'])
      ->condition('f.uri', '%static%.html', 'LIKE');
    
    // Join with node table if it exists (check with db schema api first)
    $schema = $this->database->schema();
    $node_join_available = FALSE;
    
    if ($schema->tableExists('node_field_data')) {
      $node_join_available = TRUE;
      
      // Extract node ID from URI pattern for joining
      // Use regular expression to extract node ID from paths like '/node/123'
      $query->leftJoin('node_field_data', 'n', "f.uri REGEXP :pattern", [
        ':pattern' => '.*/node/([0-9]+)\.html$',
      ]);
      $query->addField('n', 'nid');
      $query->addField('n', 'title');
      $query->addField('n', 'type');
      
      // In Drupal 11, node types are config entities, so we need to get them from config
      // rather than joining with a table
      $node_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    }
    
    // Apply filters
    $request = $this->requestStack->getCurrentRequest();
    if ($title = $request->query->get('title')) {
      if ($node_join_available) {
        $query->condition('n.title', '%' . $query->escapeLike($title) . '%', 'LIKE');
      }
    }
    
    if ($node_join_available && ($type = $request->query->get('type'))) {
      $query->condition('n.type', $type);
    }

    // Add table sort functionality
    $query = $query->extend('Drupal\Core\Database\Query\TableSortExtender')
      ->orderByHeader($header);

    // Add the pager
    $query = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')
      ->limit(25);

    $results = $query->execute()->fetchAll();
    
    // Process results to add type labels
    if ($node_join_available) {
      foreach ($results as $result) {
        if (!empty($result->type) && isset($node_types[$result->type])) {
          $result->type_label = $node_types[$result->type]->label();
        } else {
          $result->type_label = $result->type ?? $this->t('Unknown');
        }
      }
    }

    return $results;
  }

  /**
   * Format bytes to human-readable file size.
   *
   * @param int $bytes
   *   File size in bytes.
   * @param int $precision
   *   Number of decimal places.
   *
   * @return string
   *   Human-readable file size.
   */
  protected function formatBytes($bytes, $precision = 2) {
    $units = ['B', 'KB', 'MB', 'GB', 'TB'];
    
    $bytes = max($bytes, 0);
    $pow = floor(($bytes ? log($bytes) : 0) / log(1024));
    $pow = min($pow, count($units) - 1);
    
    $bytes /= (1 << (10 * $pow));
    
    return round($bytes, $precision) . ' ' . $units[$pow];
  }
}
