<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Drupal\static_node\StaticNodeHooks;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Provides a confirmation form for bulk deleting static pages.
 */
class BulkDeleteStaticConfirmForm extends ConfirmFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The static node hooks service.
   *
   * @var \Drupal\static_node\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * The node IDs to process.
   *
   * @var array
   */
  protected $nodeIds = [];

  /**
   * Constructs a new BulkDeleteStaticConfirmForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\static_node\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    MessengerInterface $messenger,
    StaticNodeHooks $static_node_hooks
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
    $this->staticNodeHooks = $static_node_hooks;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('static_node.hooks')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_bulk_delete_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $node_ids = NULL) {
    $this->nodeIds = array_filter(explode(',', $node_ids));
    
    if (empty($this->nodeIds)) {
      $this->messenger->addError($this->t('No nodes selected for static page deletion.'));
      return $this->redirect('system.admin_content');
    }
    
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $count = count($this->nodeIds);
    return $this->formatPlural(
      $count,
      'Are you sure you want to delete the static page for 1 item?',
      'Are you sure you want to delete static pages for @count items?'
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('system.admin_content');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This action will remove static HTML versions of the selected content items.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($this->nodeIds);
    $count = 0;
    $errors = 0;
    $form = [];
    
    foreach ($nodes as $node) {
      try {
        $form_state = new FormStateInterface();
        $form_state->set('node', $node);
        
        $this->staticNodeHooks->deleteStaticVersion($form, $form_state);
        $count++;
      }
      catch (\Exception $e) {
        $errors++;
        $this->messenger->addError($this->t('Error deleting static page for "@title": @message', [
          '@title' => $node->label(),
          '@message' => $e->getMessage(),
        ]));
      }
    }
    
    if ($count > 0) {
      $this->messenger->addStatus($this->formatPlural(
        $count,
        'Deleted 1 static page successfully.',
        'Deleted @count static pages successfully.'
      ));
    }
    
    if ($errors > 0) {
      $this->messenger->addWarning($this->formatPlural(
        $errors,
        'Failed to delete 1 static page. Check the logs for more details.',
        'Failed to delete @count static pages. Check the logs for more details.'
      ));
    }
    
    $form_state->setRedirectUrl($this->getCancelUrl());
  }
}
