<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Provides a confirmation form for deleting a static file.
 */
class StaticFileDeleteForm extends ConfirmFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The file ID.
   *
   * @var int
   */
  protected $fileId;

  /**
   * The file entity.
   *
   * @var \Drupal\file\FileInterface
   */
  protected $file;

  /**
   * Constructs a new StaticFileDeleteForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    MessengerInterface $messenger
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_static_file_delete_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $file = NULL) {
    // Accept either a File object or a file ID.
    if ($file instanceof \Drupal\file\FileInterface) {
      $this->fileId = $file->id();
    }
    else {
      $this->fileId = $file;
    }
    try {
      $this->file = $this->entityTypeManager->getStorage('file')->load($this->fileId);
      if (!$this->file) {
        $this->messenger->addError($this->t('The specified static file does not exist.'));
        return $this->redirect('static_node.files_management');
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('Error loading file: @error', ['@error' => $e->getMessage()]));
      return $this->redirect('static_node.files_management');
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete the static file %file?', [
      '%file' => $this->file->getFilename(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('static_node.files_management');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return new TranslatableMarkup('This action cannot be undone. The static file will be permanently deleted from the system.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    try {
      if ($this->file) {
        $filename = $this->file->getFilename();
        
        // Delete the file
        $this->file->delete();
        
        $this->messenger->addMessage($this->t('The static file %file has been deleted.', [
          '%file' => $filename,
        ]));
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('Error deleting file: @error', ['@error' => $e->getMessage()]));
    }
    
    $form_state->setRedirectUrl($this->getCancelUrl());
  }
}
