<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\static_node\StaticNodeHooks;

/**
 * Provides a confirmation form for regenerating a static file for a node.
 */
class StaticFileRegenerateForm extends ConfirmFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The static node hooks service.
   *
   * @var \Drupal\static_node\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * The node ID.
   *
   * @var int
   */
  protected $nodeId;

  /**
   * The node entity.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $node;

  /**
   * Constructs a new StaticFileRegenerateForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\static_node\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    MessengerInterface $messenger,
    StaticNodeHooks $static_node_hooks
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
    $this->staticNodeHooks = $static_node_hooks;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('static_node.hooks')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_static_file_regenerate_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $node = NULL) {
    $this->nodeId = $node;
    
    try {
      $this->node = $this->entityTypeManager->getStorage('node')->load($this->nodeId);
      
      if (!$this->node) {
        $this->messenger->addError($this->t('The specified node does not exist.'));
        return $this->redirect('static_node.files_management');
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('Error loading node: @error', ['@error' => $e->getMessage()]));
      return $this->redirect('static_node.files_management');
    }
    
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to regenerate the static file for %title?', [
      '%title' => $this->node->getTitle(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('static_node.files_management');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This will delete the existing static file and generate a new one with the latest content.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Regenerate');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->set('node', $this->node);
    
    // Delete existing static file first
    $this->staticNodeHooks->deleteStaticVersion([], $form_state);
    
    // Generate a new static file
    $this->staticNodeHooks->generateStaticVersion([], $form_state);
    
    $form_state->setRedirectUrl($this->getCancelUrl());
  }
}
