<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\static_node\StaticNodeHooks;
use Drupal\Core\TempStore\PrivateTempStoreFactory;

/**
 * Provides a confirmation form for generating a static node page.
 */
class StaticNodeGenerateForm extends ConfirmFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The static node hooks service.
   *
   * @var \Drupal\static_node\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * The tempstore service.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStore;

  /**
   * The node ID.
   *
   * @var int
   */
  protected $nodeId;

  /**
   * The node entity.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $node;

  /**
   * Constructs a new StaticNodeGenerateForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\static_node\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The temp store factory.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    MessengerInterface $messenger,
    StaticNodeHooks $static_node_hooks,
    PrivateTempStoreFactory $temp_store_factory
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
    $this->staticNodeHooks = $static_node_hooks;
    $this->tempStore = $temp_store_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('static_node.hooks'),
      $container->get('tempstore.private')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_generate_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $node = NULL) {
    // Accept either a Node object or a node ID.
    if ($node instanceof \Drupal\node\NodeInterface) {
      $this->nodeId = $node->id();
    }
    else {
      $this->nodeId = $node;
    }
    try {
      $this->node = $this->entityTypeManager->getStorage('node')->load($this->nodeId);
      if (!$this->node) {
        $this->messenger->addError($this->t('The specified node does not exist.'));
        return $this->redirect('system.admin_content');
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('Error loading node: @error', ['@error' => $e->getMessage()]));
      return $this->redirect('system.admin_content');
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to generate a static page for %title?', [
      '%title' => $this->node->getTitle(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    // Get the original destination from tempstore if available
    $tempstore = $this->tempStore->get('static_node');
    $original_destination = $tempstore->get('original_destination_' . $this->nodeId);
    
    if ($original_destination) {
      $tempstore->delete('original_destination_' . $this->nodeId);
      return Url::fromUserInput('/' . $original_destination);
    }
    
    return $this->node->toUrl();
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This will create a static HTML file for this content and set up automatic redirection to improve performance.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Generate');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->set('node', $this->node);
    
    // Generate the static file
    $this->staticNodeHooks->generateStaticVersion($form, $form_state);
    
    $form_state->setRedirectUrl($this->getCancelUrl());
  }
}
