<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Static Node settings form.
 */
class StaticNodeSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new SettingsForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['static_node.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('static_node.settings');

    // Get all node types
    $node_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $node_type_options = [];
    foreach ($node_types as $node_type) {
      $node_type_options[$node_type->id()] = $node_type->label();
    }

    $form['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General Settings'),
      '#open' => TRUE,
    ];

    $form['general']['node_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enable Static Page Generation for these content types'),
      '#description' => $this->t('Select which content types should have the Static Page Generation feature available.'),
      '#options' => $node_type_options,
      '#default_value' => $config->get('node_types') ?: [],
    ];

    $form['general']['purge_missing'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Purge static files for deleted nodes'),
      '#description' => $this->t('Automatically remove static files when the corresponding node is deleted.'),
      '#default_value' => $config->get('purge_missing') ?? TRUE,
    ];
    $form['general']['advanced_static_node'] = [
        '#type' => 'details',
        '#title' => $this->t('Advanced Settings'),
        '#open' => FALSE,
        'static_directory' => [
          '#type' => 'textfield',
          '#title' => $this->t('Static Files Directory'),
          '#description' => $this->t('Directory where static files will be stored (relative to the public files directory).'),
          '#default_value' => $config->get('static_directory') ?: 'static',
          '#required' => TRUE,
        ],
      ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('static_node.settings')
      ->set('node_types', array_filter($form_state->getValue('node_types')))
      ->set('purge_missing', $form_state->getValue('purge_missing'))
      ->set('static_directory', $form_state->getValue('static_directory'))
      ->save();

    parent::submitForm($form, $form_state);
  }
}
