<?php

namespace Drupal\static_node\Plugin\Action;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Action\ActionBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\static_node\StaticNodeHooks;

/**
 * Generates static version of a node.
 *
 * @Action(
 *   id = "static_node_generate_action",
 *   label = @Translation("Generate static page"),
 *   type = "node",
 *   confirm_form_route_name = "static_node.generate_confirm"
 * )
 */
class GenerateStaticPageAction extends ActionBase implements ContainerFactoryPluginInterface {
  use StringTranslationTrait;

  /**
   * The static node service.
   *
   * @var \Drupal\static_node\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * Constructs a GenerateStaticPageAction object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\static_node\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, StaticNodeHooks $static_node_hooks) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->staticNodeHooks = $static_node_hooks;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('static_node.hooks')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function access($node, AccountInterface $account = NULL, $return_as_object = FALSE) {
    $access = $node->access('update', $account) && $account->hasPermission('generate static node');
    return $return_as_object ? AccessResult::allowedIf($access) : $access;
  }

  /**
   * {@inheritdoc}
   */
  public function execute($node = NULL) {
    if ($node) {
      // Generate a form state object
      $form_state = new FormState();
      $form = [];
      // Set the entity on the form state
      $form_state->set('node', $node);
      
      // Call the generate method
      $this->staticNodeHooks->generateStaticVersion($form, $form_state);
      return $this->t('Generated static version for @title', ['@title' => $node->label()]);
    }
  }
}
