<?php

namespace Drupal\static_node;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\Core\Entity\EntityInterface;
use Drupal\node\Entity\Node;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\path_alias\AliasManagerInterface; 
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Cache\Cache;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use GuzzleHttp\ClientInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\file\FileRepositoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Provides hook implementations for Static Node module using attributes.
 */
class StaticNodeHooks implements ContainerInjectionInterface {
  use StringTranslationTrait;
  
  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $pathAliasManager;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The path matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;
  
  /**
   * The file repository service.
   *
   * @var \Drupal\file\FileRepositoryInterface
   */
  protected $fileRepository;
  
  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;
  
  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a new StaticNodeHooks object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\path_alias\AliasManagerInterface $path_alias_manager
   *   The path alias manager.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   The path matcher.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\file\FileRepositoryInterface $file_repository
   *   The file repository service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    AliasManagerInterface $path_alias_manager,
    FileSystemInterface $file_system,
    FileUrlGeneratorInterface $file_url_generator,
    AccountProxyInterface $current_user,
    LanguageManagerInterface $language_manager,
    ModuleHandlerInterface $module_handler,
    ClientInterface $http_client,
    ConfigFactoryInterface $config_factory,
    PathMatcherInterface $path_matcher,
    RequestStack $request_stack,
    FileRepositoryInterface $file_repository,
    MessengerInterface $messenger,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->pathAliasManager = $path_alias_manager;
    $this->fileSystem = $file_system;
    $this->fileUrlGenerator = $file_url_generator;
    $this->currentUser = $current_user;
    $this->languageManager = $language_manager;
    $this->moduleHandler = $module_handler;
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->pathMatcher = $path_matcher;
    $this->requestStack = $request_stack;
    $this->fileRepository = $file_repository;
    $this->messenger = $messenger;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('path_alias.manager'),
      $container->get('file_system'),
      $container->get('file_url_generator'),
      $container->get('current_user'),
      $container->get('language_manager'),
      $container->get('module_handler'),
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('path.matcher'),
      $container->get('request_stack'),
      $container->get('file.repository'),
      $container->get('messenger'),
      $container->get('logger.factory')
    );
  }

  /**
   * Implements hook_form_node_form_alter().
   */
  #[Hook('form_node_form_alter')]
  public function formNodeFormAlter(array &$form, FormStateInterface $form_state, $form_id) {
    $node = $form_state->getFormObject()->getEntity();
    if (!$node instanceof Node) {
      return;
    }
    $nid = $node->id();
    if (!$nid) {
      return;
    }
    
    // Check if user has the permission to generate static nodes
    if (!$this->currentUser->hasPermission('generate static node')) {
      return;
    }
    
    // Check if this node type is enabled for static generation
    $enabled_types = $this->configFactory->get('static_node.settings')->get('node_types') ?: [];
    if (!in_array($node->bundle(), $enabled_types)) {
      return;
    }
    
    $alias = $this->pathAliasManager->getAliasByPath('/node/' . $nid);
    $locale = $this->languageManager->getCurrentLanguage()->getId();

    // Place the form element directly in the main form rather than in a subgroup
    $form['static_page'] = [
      '#type' => 'details',
      '#title' => $this->t('Static Page Generator'),
      '#description' => $this->t('Generate a static HTML file for this node and automatically redirect the page for improved performance.'),
      '#open' => TRUE,
      '#group' => 'advanced',  // This connects it to the right sidebar
      '#attributes' => ['class' => ['generate-static-version-form']],
      '#weight' => -100,
    ];

    $form['static_page']['actions'] = [
      '#type' => 'container',
    ];

    // Update the generate button to use a confirmation form
    $form['static_page']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Generate Static Page'),
      '#submit' => [[$this, 'generateStaticVersionConfirm']],
      '#button_type' => 'primary',
    ];

    // Check if a static file exists for this node
    $static_file = $this->findStaticFile($locale, $alias);
    
    if ($static_file) {
      $file_url = $this->fileUrlGenerator->generateAbsoluteString($static_file->uri->value);
      
      $this->messenger->deleteByType('warning');
      $this->messenger->addWarning($this->t('This node has static version configured'));
      
      // Only show delete button if user has delete permission
      if ($this->currentUser->hasPermission('delete static node')) {
        $form['static_page']['actions']['delete'] = [
          '#type' => 'submit',
          '#value' => $this->t('Delete Static Page'),
          '#submit' => [[$this, 'deleteStaticVersionConfirm']],
          '#button_type' => 'secondary',
        ];
      }
      
      $form['static_page']['actions']['submit']['#value'] = $this->t('Update Static Page');
      $form['static_page']['#open'] = TRUE;
      
      // Make sure there is a description before appending to it
      if (!isset($form['static_page']['#description'])) {
        $form['static_page']['#description'] = '';
      }
      
      $form['static_page']['#description'] .= '<br>' . $this->t('Static HTML page URL:') . '<br><b>' . $file_url . '</b>';
      
      // Also append to the main submit
      $form['actions']['submit']['#submit'][] = [$this, 'generateStaticVersionConfirm'];
    }
  }
  
  /**
   * Submit handler to redirect to the confirmation form for static page generation.
   */
  public function generateStaticVersionConfirm(array &$form, FormStateInterface $form_state): void {
    $node = $form_state->getFormObject()->getEntity();
    if (!$node instanceof Node || !$node->id()) {
      return;
    }
    
    // Get the current path to use as a destination after the confirmation
    $current_path = \Drupal::request()->getRequestUri();
    
    // Store the original destination in a session to be used after confirmation
    $request = \Drupal::request();
    if ($request->query->has('destination')) {
      $original_destination = $request->query->get('destination');
      $request->query->remove('destination');
      
      // Store it in session for later use
      $tempstore = \Drupal::service('tempstore.private')->get('static_node');
      $tempstore->set('original_destination_' . $node->id(), $original_destination);
    }
    
    // Build the URL for the confirmation form
    $url = Url::fromRoute('static_node.generate_confirm', ['node' => $node->id()]);
    Cache::invalidateTags($node->getCacheTags());
    // Set the redirect without the problematic destination parameter
    $form_state->setRedirectUrl($url);
  }

  /**
   * Submit handler to redirect to the confirmation form for static page deletion.
   */
  public function deleteStaticVersionConfirm(array &$form, FormStateInterface $form_state): void {
    $node = $form_state->getFormObject()->getEntity();
    if (!$node instanceof Node || !$node->id()) {
      return;
    }
    
    // Store the original destination in a session to be used after confirmation
    $request = \Drupal::request();
    if ($request->query->has('destination')) {
      $original_destination = $request->query->get('destination');
      $request->query->remove('destination');
      
      // Store it in session for later use
      $tempstore = \Drupal::service('tempstore.private')->get('static_node');
      $tempstore->set('original_destination_' . $node->id(), $original_destination);
    }
    
    // Build the URL for the confirmation form
    $url = Url::fromRoute('static_node.delete_confirm', ['node' => $node->id()]);
    
    Cache::invalidateTags($node->getCacheTags());
    // Set the redirect without the problematic destination parameter
    $form_state->setRedirectUrl($url);
  }

  /**
   * Implements hook_preprocess().
   */
  #[Hook('preprocess')]
  public function preprocess(&$variables, $hook) {
    // Only process for node and page hooks
    if ($hook !== 'node' && $hook !== 'page') {
      return;
    }
    
    // Check if the user is authenticated (no redirect for authenticated users)
    if ($this->currentUser->isAuthenticated()) {
      return;
    }
    
    // Check if we already redirected to avoid endless loops
    $redirected = &drupal_static('static_node_redirected', FALSE);
    if ($redirected) {
      return;
    }

    $current_request = $this->requestStack->getCurrentRequest();
    $current_path = $current_request->getPathInfo();
    
    // Special handling for homepage
    if ($current_path === '/' || $this->pathMatcher->isFrontPage()) {
      // Try to find the static version of the homepage
      $static_file = $this->findHomepageStaticFile();
      
      if ($static_file) {
        $file_url = $this->fileUrlGenerator->generateAbsoluteString($static_file->uri->value);
        
        // Mark as redirected to avoid endless loops
        $redirected = TRUE;
        
        // Redirect to the static file
        $response = new RedirectResponse($file_url, 302);
        $response->send();
        exit;
      }
      
      // If no static homepage found, just return and let Drupal serve the dynamic page
      return;
    }
    
    // For non-homepage content, try to get the node from different template variables
    $node = NULL;
    
    if (isset($variables['node']) && $variables['node'] instanceof EntityInterface) {
      $node = $variables['node'];
    }
    else {
      // Try to get node from route context
      $route_match = \Drupal::routeMatch();
      $node = $route_match->getParameter('node');
    }
    
    // If we couldn't find a node, exit early
    if (!$node instanceof EntityInterface) {
      return;
    }
    
    $nid = $node->id();
    $locale = $this->languageManager->getCurrentLanguage()->getId();
    $alias = $this->pathAliasManager->getAliasByPath('/node/' . $nid);
    
    // Try multiple methods to find the static file
    $static_file = $this->findStaticFile($locale, $alias);
    
    if (!$static_file) {
      return;
    }
    
    $file_url = $this->fileUrlGenerator->generateAbsoluteString($static_file->uri->value);
    
    // Extract just the path from the file URL for comparison
    $file_url_parsed = parse_url($file_url);
    $file_path = $file_url_parsed['path'] ?? '';
    
    // Don't redirect if on admin pages
    if (strpos($current_path, '/admin') === 0) {
      return;
    }
    
    // Compare the current URL path with the static file path
    $current_url = $current_request->getUri();
    $current_url_parsed = parse_url($current_url);
    $current_path = $current_url_parsed['path'] ?? '';
    
    // If we're already on the static file, don't redirect
    if ($current_path === $file_path) {
      return;
    }
    
    // Check if this is a callback or AJAX request that shouldn't be redirected
    if ($current_request->isXmlHttpRequest() || $current_request->query->has('_wrapper_format')) {
      return;
    }
    
    // Mark as redirected to avoid endless loops
    $redirected = TRUE;
    
    // Always invalidate cache for this node
    Cache::invalidateTags($node->getCacheTags());
    
    // Redirect to the static file
    $response = new RedirectResponse($file_url, 302);
    $response->send();
    exit;
  }
  
  /**
   * Find the static file for the homepage.
   *
   * @return \Drupal\file\FileInterface|null
   *   The static file entity or NULL if not found.
   */
  protected function findHomepageStaticFile() {
    $locale = $this->languageManager->getCurrentLanguage()->getId();
    $file_storage = $this->entityTypeManager->getStorage('file');
    
    // Check for common homepage patterns
    $homepage_paths = [
      "public://static/{$locale}/index.html",
      "public://static/{$locale}/front.html",
      "public://static/{$locale}/home.html",
      "public://static/index.html",
      "public://static/front.html", 
      "public://static/home.html",
    ];
    
    // Get the node ID of the front page if available
    try {
      $config = $this->configFactory->get('system.site');
      $front_page = $config->get('page.front');
      
      if (preg_match('/node\/(\d+)/', $front_page, $matches)) {
        $front_nid = $matches[1];
        $alias = $this->pathAliasManager->getAliasByPath('/node/' . $front_nid);
        
        if ($alias) {
          $homepage_paths[] = "public://static/{$locale}{$alias}.html";
        }
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('static_node')->warning('Could not determine front page node: @error', [
        '@error' => $e->getMessage(),
      ]);
    }
    
    // Try each potential homepage path
    foreach ($homepage_paths as $path) {
      $files = $file_storage->loadByProperties(['uri' => $path]);
      if (!empty($files)) {
        return reset($files);
      }
    }
    
    // Try a more generic approach with a query 
    try {
      $query = $file_storage->getQuery()
        ->condition('uri', 'public://static/%index%.html', 'LIKE')
        ->accessCheck(FALSE)  // Explicitly disable access checking for system operation
        ->range(0, 1);
      $fids = $query->execute();
      
      if (!empty($fids)) {
        $files = $file_storage->loadMultiple($fids);
        return reset($files);
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('static_node')->error('Error finding home page static file: @error', [
        '@error' => $e->getMessage(),
      ]);
    }
    
    return NULL;
  }

  /**
   * Public method to access homepage static file finder.
   *
   * @return \Drupal\file\FileInterface|null
   *   The static file entity or NULL if not found.
   */
  public function getHomepageStaticFile() {
    return $this->findHomepageStaticFile();
  }

  /**
   * Generates a static version of the homepage.
   */
  public function generateStaticHomepage(): void {
    // Check if user has the permission to generate static nodes
    if (!$this->currentUser->hasPermission('generate static node')) {
      $this->messenger->addError($this->t('You do not have permission to generate static pages.'));
      return;
    }
    
    $locale = $this->languageManager->getCurrentLanguage()->getId() ?: '';
    $file_path = "public://static/{$locale}/index.html";
    $file_directory = dirname($file_path);
    
    if (!$this->fileSystem->prepareDirectory($file_directory, FileSystemInterface::CREATE_DIRECTORY)) {
      $this->messenger->addError($this->t('Unable to create directory.'));
      return;
    }
    
    // Clean up any existing file first
    $static_file = $this->findHomepageStaticFile();
    if ($static_file) {
      $static_file->delete();
    }
    
    $options = [];
    
    // Handle Shield module if enabled
    if ($this->moduleHandler->moduleExists('shield')) {
      $shield_config = $this->configFactory->get('shield.settings');
      if ($shield_config && $shield_config->get('shield_enable')) {
        $shield_credential = $shield_config->get('credentials.shield');
        $user = $shield_credential["user"] ?? '';
        $pass = $shield_credential["pass"] ?? '';
        if ($user && $pass) {
          $options['headers'] = ['Authorization' => 'Basic ' . base64_encode($user . ':' . $pass)];
        }
      }
    }

    // Get the base URL for the homepage (front page)
    $base_url = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost();
    $url = $base_url . "/";
    
    try {
      $options['verify'] = false;
      $response = $this->httpClient->request('GET', $url, $options);

      if ($response->getStatusCode() === 200) {
        $html = $response->getBody()->getContents();

        // Load content into a DOM object
        $dom = new \DOMDocument();
        // Suppress warnings for malformed HTML
        @$dom->loadHTML($html);

        // Process CSS files
        $this->processCssFiles($dom, $base_url, $locale, '/index', $options);
        
        // Process script files
        $this->processScriptFiles($dom, $base_url, $options);
        
        // Process JSON files
        $this->processJsonFiles($dom, $base_url, $options);
        
        // Process favicons
        $this->processFavicons($dom, $base_url, $options);
        
        // Process all links to make them absolute
        $this->processLinks($dom, $base_url);
        
        // Process buttons with onclick handlers
        $this->processButtons($dom, $base_url);

        $updated_html = $dom->saveHTML();

        $file = $this->fileRepository->writeData($updated_html, $file_path, FileSystemInterface::EXISTS_REPLACE);
        
        if ($file) {
          $this->messenger->addMessage($this->t('Static homepage generated successfully.'));
        }
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('Error generating static homepage: @message', ['@message' => $e->getMessage()]));
      $this->loggerFactory->get('static_node')->error('Error generating static homepage: @message', ['@message' => $e->getMessage()]);
    }
  }
  
  /**
   * Deletes the static version of the homepage.
   */
  public function deleteStaticHomepage(): void {
    // Check if user has the permission to delete static nodes
    if (!$this->currentUser->hasPermission('delete static node')) {
      $this->messenger->addError($this->t('You do not have permission to delete static pages.'));
      return;
    }
    
    $static_file = $this->findHomepageStaticFile();
    
    if ($static_file) {
      $static_file->delete();
      $this->messenger->addStatus($this->t('Static homepage file deleted.'));
      Cache::invalidateTags(['config:system.site']);
    } 
    else {
      $this->messenger->addStatus($this->t('No static homepage file found to delete.'));
    }
  }

  /**
   * Find a static file for a given path.
   *
   * @param string $locale
   *   The language locale.
   * @param string $alias
   *   The path alias.
   *
   * @return \Drupal\file\FileInterface|null
   *   The static file entity or NULL if not found.
   */
  protected function findStaticFile(string $locale, string $alias) {
    $file_storage = $this->entityTypeManager->getStorage('file');
    
    // 1. Try exact path match first
    $file_path = "public://static/{$locale}{$alias}.html";
    $files = $file_storage->loadByProperties(['uri' => $file_path]);
    if (!empty($files)) {
      return reset($files);
    }
    
    // 2. Try by filename for the homepage.
    $file_name = str_replace('/', '', $alias) . '.html';
    $files = $file_storage->loadByProperties(['filename' => $file_name]);
    if (!empty($files)) {
      return reset($files);
    }
    
    // 3. Try with a SQL LIKE query
    try {
      // Use LIKE query to find files with URI containing the path segment
      $query = $file_storage->getQuery()
        ->condition('uri', '%' . $alias . '.html', 'LIKE')
        ->accessCheck(FALSE);
      $fids = $query->execute();
      if (!empty($fids)) {
        $files = $file_storage->loadMultiple($fids);
        foreach ($files as $file) {
          // Check if the file URI contains the locale
          if (strpos($file->getFileUri(), "/{$locale}/") !== FALSE) {
            return $file;
          }
        }
        // If no file with locale found, just return the first one
        return reset($files);
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('static_node')->error('Error finding static file: @error', [
        '@error' => $e->getMessage(),
      ]);
    }
    
    return NULL;
  }

  /**
   * Ensure the field_static_node entity reference field exists on File entity.
   */
  protected function ensureStaticNodeFieldOnFile() {
    $field_name = 'field_static_node';
    $entity_type = 'file';
    $bundle = 'file';
    $field_storage = $this->entityTypeManager->getStorage('field_storage_config');
    $field_config = $this->entityTypeManager->getStorage('field_config');

    // Check if the field storage exists.
    $storage = $field_storage->load($entity_type . '.' . $field_name);
    if (!$storage) {
      $storage = $field_storage->create([
        'field_name' => $field_name,
        'entity_type' => $entity_type,
        'type' => 'entity_reference',
        'settings' => [
          'target_type' => 'node',
        ],
        'cardinality' => 1,
      ]);
      $storage->save();
    }

    // Check if the field config exists for the default bundle.
    $config = $field_config->load($entity_type . '.' . $bundle . '.' . $field_name);
    if (!$config) {
      $config = $field_config->create([
        'field_name' => $field_name,
        'entity_type' => $entity_type,
        'bundle' => $bundle,
        'label' => 'Source Node',
        'settings' => [
          'handler' => 'default',
          'handler_settings' => [
            'target_bundles' => NULL,
          ],
        ],
        'required' => FALSE,
      ]);
      $config->save();
    }
  }

  /**
   * Handles Static Page generation.
   */
  public function generateStaticVersion(array &$form, FormStateInterface $form_state): void {
    // Check if user has the permission to generate static nodes
    if (!$this->currentUser->hasPermission('generate static node')) {
      $this->messenger->addError($this->t('You do not have permission to generate static pages.'));
      return;
    }
    
    // Try to get node from form state directly (for bulk actions)
    $node = $form_state->get('node');
    // If not found, try to get from form object (for normal form submission)
    if (!$node instanceof Node && $form_state->getFormObject()) {
      $node = $form_state->getFormObject()->getEntity();
    }
    
    if (!$node instanceof Node) {
      $this->messenger->addError($this->t('Invalid node type.'));
      return;
    }
    
    $nid = $node->id();
    if (!$nid) {
      $this->messenger->addError($this->t('Node not found.'));
      return;
    }
    
    $locale = $this->languageManager->getCurrentLanguage()->getId() ?: '';
    $alias = $this->pathAliasManager->getAliasByPath('/node/' . $nid);
    $file_path = "public://static/{$locale}{$alias}.html";
    $file_directory = dirname($file_path);
    
    if (!$this->fileSystem->prepareDirectory($file_directory, FileSystemInterface::CREATE_DIRECTORY)) {
      $this->messenger->addError($this->t('Unable to create directory.'));
      return;
    }
    
    $file_name = str_replace('/', '', $alias) . '.html';
    $target_file = $this->entityTypeManager->getStorage('file')
      ->loadByProperties(['filename' => $file_name]);
    
    if ($file = reset($target_file)) {
      $file->delete();
    }
    
    $options = [];
    if ($this->moduleHandler->moduleExists('shield')) {
      $shield_config = $this->configFactory->get('shield.settings');
      if ($shield_config && $shield_config->get('shield_enable')) {
        $shield_credential = $shield_config->get('credentials.shield');
        $user = $shield_credential["user"] ?? '';
        $pass = $shield_credential["pass"] ?? '';
        if ($user && $pass) {
          $options['headers'] = ['Authorization' => 'Basic ' . base64_encode($user . ':' . $pass)];
        }
      }
    }
    
    if ($locale && !empty($locale)) {
      $url = Url::fromRoute('entity.node.canonical', ['node' => $nid], [
        'absolute' => TRUE,
        'language' => $this->languageManager->getCurrentLanguage()
      ])->toString();
    } else {
      $url = Url::fromRoute('entity.node.canonical', ['node' => $nid], [
        'absolute' => TRUE
      ])->toString();
    }
    
    try {
      $options['verify'] = false;
      $response = $this->httpClient->request('GET', $url, $options);

      if ($response->getStatusCode() === 200) {
        $html = $response->getBody()->getContents();

        // Extract base URL from the original URL
        $parsed_url = parse_url($url);
        $base_url = $parsed_url['scheme'] . '://' . $parsed_url['host'];
        if (isset($parsed_url['port'])) {
          $base_url .= ':' . $parsed_url['port'];
        }

        // Load content into a DOM object
        $dom = new \DOMDocument();
        // Suppress warnings for malformed HTML
        @$dom->loadHTML($html);

        // Process CSS files
        $this->processCssFiles($dom, $base_url, $locale, $alias, $options);
        
        // Process script files
        $this->processScriptFiles($dom, $base_url, $options);
        
        // Process JSON files
        $this->processJsonFiles($dom, $base_url, $options);
        
        // Process favicons
        $this->processFavicons($dom, $base_url, $options);
        
        // Process all links to make them absolute
        $this->processLinks($dom, $base_url);
        
        // Process buttons with onclick handlers
        $this->processButtons($dom, $base_url);

        $updated_html = $dom->saveHTML();
    
        $file = $this->fileRepository->writeData($updated_html, $file_path, FileSystemInterface::EXISTS_REPLACE);
        // Relate static file to node
        $this->ensureStaticNodeFieldOnFile();
        if ($file && $file->hasField('field_static_node')) {
          $file->set('field_static_node', $nid);
          $file->save();
        }
        if ($file) {
          $this->messenger->addMessage($this->t('Static Page generated successfully.'));
        }
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('Error generating static page: @message', ['@message' => $e->getMessage()]));
      $this->loggerFactory->get('static_node')->error('Error generating static page: @message', ['@message' => $e->getMessage()]);
    }
  } 

  /**
   * Handles Static Page deletion.
   */
  public function deleteStaticVersion(array &$form, FormStateInterface $form_state): void {
    // Check if user has the permission to delete static nodes
    if (!$this->currentUser->hasPermission('delete static node')) {
      $this->messenger->addError($this->t('You do not have permission to delete static pages.'));
      return;
    }
    
    // Try to get node from form state directly (for bulk actions)
    $node = $form_state->get('node');
    // If not found, try to get from form object (for normal form submission)
    if (!$node instanceof Node && $form_state->getFormObject()) {
      $node = $form_state->getFormObject()->getEntity();
    }
    
    if (!$node instanceof Node) {
      $this->messenger->addError($this->t('Invalid node type.'));
      return;
    }
    
    $nid = $node->id();
    $locale = $this->languageManager->getCurrentLanguage()->getId() ?: '';
    $alias = $this->pathAliasManager->getAliasByPath('/node/' . $nid);
    
    // Find and delete the static file
    $static_file = $this->findStaticFile($locale, $alias);
    if ($static_file) {
      $static_file->delete();
      $this->messenger->addStatus($this->t('Static Page file deleted.'));
    } 
    else {
      $this->messenger->addStatus($this->t('No Static Page file found to delete.'));
    }
  }

  /**
   * Remove query string from URL.
   */
  protected function removeQueryString(string $url): string {
    return strtok($url, '?');
  }

  /**
   * Process CSS files in a DOM document.
   */
  protected function processCssFiles(\DOMDocument $dom, string $base_url, string $locale, string $alias, array $options): void {
    $links = $dom->getElementsByTagName('link');
    foreach ($links as $link) {
      $rel = strtolower($link->getAttribute('rel'));
      $as = strtolower($link->getAttribute('as'));

      // CSS handling
      if ($rel === 'stylesheet') {
        $options['headers']['Accept'] = 'text/css,*/*;q=0.1';
        $css_url = $link->getAttribute('href');
        
        if (strpos($css_url, '/sites/default') === 0 || strpos($css_url, '/themes/custom') === 0 || strpos($css_url, '/modules') === 0) {
          $css_url = $base_url . $css_url;
        } else {
          continue;
        }

        try {
          $css_response = $this->httpClient->request('GET', $css_url, $options);
          $css_content = $css_response->getBody()->getContents();
        
          // Update font references in CSS
          $css_content = preg_replace_callback('/url\(([^)]+)\)/', function ($matches) use ($base_url) {
            $font_url = trim($matches[1], '"\'');
            // If the font URL is relative, update it to the new path
            if (strpos($font_url, '/themes/custom') === 0 || strpos($font_url, '/modules') === 0) {
              $font_url = $base_url . $font_url;
            }
            // Modify the font path
            $new_font_url = $this->removeQueryString($font_url);
            return 'url(' . $new_font_url . ')';
          }, $css_content);

          $css_file_path = "public://static/{$locale}{$alias}-" . basename($this->removeQueryString($css_url));
          $css_file_directory = dirname($css_file_path);

          if (!$this->fileSystem->prepareDirectory($css_file_directory, FileSystemInterface::CREATE_DIRECTORY)) {
            $this->messenger->addError($this->t('Unable to create directory.'));
            return;
          }

          $this->fileRepository->writeData($css_content, $css_file_path, FileSystemInterface::EXISTS_REPLACE);
          // Update the link in the DOM
          $css_file_url = $this->fileUrlGenerator->generateAbsoluteString($css_file_path);
          $link->setAttribute('href', $this->removeQueryString($css_file_url));
        } catch (\Exception $e) {
          $this->loggerFactory->get('static_node')->error('Error retrieving CSS: @url - @error', [
            '@url' => $css_url,
            '@error' => $e->getMessage(),
          ]);
        }
      }
    
      // Font handling (preload)
      if ($rel === 'preload' && $as === 'font') {
        $options['headers']['Accept'] = 'font/woff2, font/woff, */*;q=0.1';
        $font_url = $link->getAttribute('href');
    
        if (strpos($font_url, '/themes/custom') === 0 || strpos($font_url, '/modules') === 0) {
          $font_url = $base_url . $font_url;
        } else {
          continue;
        }

        try {
          $font_response = $this->httpClient->request('GET', $font_url, $options);
          $font_content = $font_response->getBody()->getContents();
          $font_file_path = "public://static/fonts/" . basename($this->removeQueryString($font_url));
          $font_file_directory = dirname($font_file_path);

          if (!$this->fileSystem->prepareDirectory($font_file_directory, FileSystemInterface::CREATE_DIRECTORY)) {
            $this->messenger->addError($this->t('Unable to create directory for fonts.'));
            return;
          }

          $this->fileRepository->writeData($font_content, $font_file_path, FileSystemInterface::EXISTS_REPLACE);
          // Update the link in the DOM with the absolute path of the font
          $font_file_url = $this->fileUrlGenerator->generateAbsoluteString($font_file_path);
          $link->setAttribute('href', $this->removeQueryString($font_file_url));
        } catch (\Exception $e) {
          $this->loggerFactory->get('static_node')->error('Error retrieving font: @url - @error', [
            '@url' => $font_url,
            '@error' => $e->getMessage(),
          ]);
        }
      }
    }
  }

  /**
   * Process script files in a DOM document.
   */
  protected function processScriptFiles(\DOMDocument $dom, string $base_url, array $options): void {
    $scripts = $dom->getElementsByTagName('script');
    
    foreach ($scripts as $script) {
      $options['headers']['Accept'] = 'application/javascript, text/javascript, */*;q=0.1';
      if ($script && $script->getAttribute('src')) {
        $js_url = $script->getAttribute('src');
        
        if (strpos($js_url, '/sites/default') === 0 || strpos($js_url, '/themes/custom') === 0 || strpos($js_url, '/modules') === 0) {
          // Fix: Use string concatenation operator (.) instead of addition (+)
          $js_url = $base_url . $js_url;
        } else {
          continue;
        }

        try {
          $js_response = $this->httpClient->request('GET', $js_url, $options);
          $js_content = $js_response->getBody()->getContents();
          $js_file_path = "public://static/js/" . basename($this->removeQueryString($js_url));
          $js_file_directory = dirname($js_file_path);

          if (!$this->fileSystem->prepareDirectory($js_file_directory, FileSystemInterface::CREATE_DIRECTORY)) {
            $this->messenger->addError($this->t('Unable to create directory for JavaScript.'));
            return;
          }

          // Analyze JS content to find internal resource references
          preg_match_all('/["\'](\/modules\/[^"\']+)["\']/', $js_content, $matches);
          foreach ($matches[1] as $resource_url) {
            $full_resource_url = $base_url . $resource_url;
            $resource_filename = basename($this->removeQueryString($resource_url));
            $resource_file_path = "public://static/modules/" . $resource_filename;

            try {
              $resource_response = $this->httpClient->request('GET', $full_resource_url, $options);
              $resource_content = $resource_response->getBody()->getContents();

              $resource_directory = dirname($resource_file_path);
              if (!$this->fileSystem->prepareDirectory($resource_directory, FileSystemInterface::CREATE_DIRECTORY)) {
                $this->messenger->addError($this->t('Unable to create directory for module resources.'));
                return;
              }

              $this->fileRepository->writeData($resource_content, $resource_file_path, FileSystemInterface::EXISTS_REPLACE);
              // Update references in JavaScript
              $local_resource_url = $this->fileUrlGenerator->generateAbsoluteString($resource_file_path);
              $js_content = str_replace($resource_url, $this->removeQueryString($local_resource_url), $js_content);
            } catch (\Exception $e) {
              $this->loggerFactory->get('static_node')->error('Error retrieving JS resource: @url - @error', [
                '@url' => $full_resource_url,
                '@error' => $e->getMessage(),
              ]);
            }
          }

          // Save the updated JavaScript file
          $this->fileRepository->writeData($js_content, $js_file_path, FileSystemInterface::EXISTS_REPLACE);
          // Update the link in the DOM
          $js_file_url = $this->fileUrlGenerator->generateAbsoluteString($js_file_path);
          $script->setAttribute('src', $this->removeQueryString($js_file_url));
        } catch (\Exception $e) {
          $this->loggerFactory->get('static_node')->error('Error retrieving JS: @url - @error', [
            '@url' => $js_url,
            '@error' => $e->getMessage(),
          ]);
        }
      }
    }
  }

  /**
   * Process JSON files in a DOM document.
   */
  protected function processJsonFiles(\DOMDocument $dom, string $base_url, array $options): void {
    $json_scripts = $dom->getElementsByTagName('script');

    foreach ($json_scripts as $script) {
      if ($script->getAttribute('type') === 'application/json' && $script->hasAttribute('src')) {
        $json_url = $script->getAttribute('src');

        if (strpos($json_url, '/sites/default') === 0) {
          $json_url = $base_url . $json_url;
        } else {
          continue;
        }

        try {
          $json_response = $this->httpClient->request('GET', $json_url, $options);
          $json_content = $json_response->getBody()->getContents();
          $json_file_path = "public://static/json/" . basename($json_url);
          
          $json_directory = dirname($json_file_path);
          if (!$this->fileSystem->prepareDirectory($json_directory, FileSystemInterface::CREATE_DIRECTORY)) {
            $this->messenger->addError($this->t('Unable to create directory for JSON files.'));
            return;
          }

          $this->fileRepository->writeData($json_content, $json_file_path, FileSystemInterface::EXISTS_REPLACE);
          $json_file_url = $this->fileUrlGenerator->generateAbsoluteString($json_file_path);
          $script->setAttribute('src', $this->removeQueryString($json_file_url));
        } catch (\Exception $e) {
          $this->loggerFactory->get('static_node')->error('Error retrieving JSON: @url - @error', [
            '@url' => $json_url,
            '@error' => $e->getMessage(),
          ]);
        }
      }
    }
  }

  /**
   * Process favicons in a DOM document.
   */
  protected function processFavicons(\DOMDocument $dom, string $base_url, array $options): void {
    $favicons = $dom->getElementsByTagName('link');

    foreach ($favicons as $favicon) {
      if (in_array(strtolower($favicon->getAttribute('rel')), ['icon', 'shortcut icon'])) {
        $favicon_url = $favicon->getAttribute('href');

        if (strpos($favicon_url, '/sites/default') === 0) {
          // Fix: Use string concatenation operator (.) instead of addition (+)
          $favicon_url = $base_url . $favicon_url;
        } else {
          continue;
        }

        try {
          $favicon_response = $this->httpClient->request('GET', $favicon_url, $options);
          $favicon_content = $favicon_response->getBody()->getContents();
          $favicon_file_path = "public://static/" . basename($favicon_url);
          
          $favicon_directory = dirname($favicon_file_path);
          if (!$this->fileSystem->prepareDirectory($favicon_directory, FileSystemInterface::CREATE_DIRECTORY)) {
            $this->messenger->addError($this->t('Unable to create directory for favicons.'));
            return;
          }

          $this->fileRepository->writeData($favicon_content, $favicon_file_path, FileSystemInterface::EXISTS_REPLACE);

          $favicon_file_url = $this->fileUrlGenerator->generateAbsoluteString($favicon_file_path);
          $favicon->setAttribute('href', $this->removeQueryString($favicon_file_url));
        } catch (\Exception $e) {
          $this->loggerFactory->get('static_node')->error('Error retrieving favicon: @url - @error', [
            '@url' => $favicon_url,
            '@error' => $e->getMessage(),
          ]);
        }
      }
    }
  }

  /**
   * Process links in a DOM document to make them absolute.
   */
  protected function processLinks(\DOMDocument $dom, string $base_url): void {
    $links = $dom->getElementsByTagName('a');

    foreach ($links as $link) {
      $href = $link->getAttribute('href');

      // Check if the URL is relative (starts with "/")
      if (strncmp($href, '/', 1) === 0 && strncmp($href, '//', 2) !== 0) {
        // Build the absolute path by concatenating $base_url with the relative path
        $absolute_url = $base_url . $href;
        $link->setAttribute('href', str_replace('/s3fs-public/static', '', $absolute_url));
      }
    }
  }

  /**
   * Process buttons with onclick handlers in a DOM document.
   */
  protected function processButtons(\DOMDocument $dom, string $base_url): void {
    $buttons = $dom->getElementsByTagName('*');
    foreach ($buttons as $button) {
      $onclick = $button->getAttribute('onclick');

      // If the onclick attribute contains a relative URL, modify the URL
      if (strpos($onclick, "redirectToUrl('") !== false) {
        // Find the URL inside the redirectToUrl() function
        if (preg_match("/redirectToUrl\('([^']+)'\)/", $onclick, $matches)) {
          $relative_url = $matches[1];

          // Check if the URL is relative
          if (strncmp($relative_url, '/', 1) === 0 && strncmp($relative_url, '//', 2) !== 0) {
            // Create the absolute path
            $absolute_url = $base_url . $relative_url;
            // Replace the relative URL with the absolute URL
            $onclick = str_replace($relative_url, $absolute_url, $onclick);
            $button->setAttribute('onclick', str_replace('/s3fs-public/static', '', $onclick));
          }
        }
      }

      // Handle the case: onclick="window.location.href = 'en/livigno'"
      if (strpos($onclick, "window.location.href = '") !== false) {
        // Find the URL inside the window.location.href function
        if (preg_match("/window.location.href = '([^']+)'/", $onclick, $matches)) {
          $relative_url = $matches[1];

          // Check if the URL is relative
          if (strncmp($relative_url, '/', 1) === 0 && strncmp($relative_url, '//', 2) !== 0) {
            // Create the absolute path
            $absolute_url = $base_url . $relative_url;
            // Replace the relative URL with the absolute URL
            $onclick = str_replace($relative_url, $absolute_url, $onclick);
            $button->setAttribute('onclick', str_replace('/s3fs-public/static', '', $onclick));
          }
        }
      }
    }
  }
}
